@@ -1,10 +1,10 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: client.c,v 1.57 2002/10/09 15:57:03 rjs3 Exp $
+ * $Id: client.c,v 1.60 2003/03/19 18:25:27 rjs3 Exp $
  */
 /* 
- * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -210,7 +210,7 @@ int sasl_client_init(const sasl_callback_t *callbacks)
 
   sasl_client_add_plugin("EXTERNAL", &external_client_plug_init);
 
-  ret = _sasl_common_init();
+  ret = _sasl_common_init(&global_callbacks);
 
   if (ret == SASL_OK)
       ret = _sasl_load_plugins(ep_list,
@@ -324,6 +324,7 @@ int sasl_client_new(const char *service,
   conn->cparams->utils = utils;
   conn->cparams->canon_user = &_sasl_canon_user;
   conn->cparams->flags = flags;
+  conn->cparams->prompt_supp = (*pconn)->callbacks;
   
   /* get the clientFQDN (serverFQDN was set in _sasl_conn_init) */
   memset(name, 0, sizeof(name));
@@ -633,7 +634,7 @@ int sasl_client_step(sasl_conn_t *conn,
 					 serverin,
 					 serverinlen,
 					 prompt_need,
-					 clientout, (int *)clientoutlen,
+					 clientout, clientoutlen,
 					 &conn->oparams);
 
   if (result == SASL_OK) {
