@@ -1,9 +1,10 @@
+
 /* Generic SASL plugin utility functions
  * Rob Siemborski
- * $Id: plugin_common.h,v 1.14 2002/12/09 06:08:50 leg Exp $
+ * $Id: plugin_common.h,v 1.16 2003/04/07 16:03:43 rjs3 Exp $
  */
 /* 
- * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -98,6 +99,16 @@ PLUG_API int sasl_auxprop_plug_init(const sasl_utils_t *utils, \
                                      plug, plugname); \
 }
 
+#define SASL_CANONUSER_PLUG_INIT( x ) \
+extern sasl_canonuser_init_t x##_canonuser_plug_init; \
+PLUG_API int sasl_canonuser_init(const sasl_utils_t *utils, \
+                           int maxversion, int *out_version, \
+                           sasl_canonuser_plug_t **plug, \
+                           const char *plugname) {\
+        return x##_canonuser_plug_init(utils, maxversion, out_version, \
+                                     plug, plugname); \
+}
+
 /* note: msg cannot include additional variables, so if you want to
  * do a printf-format string, then you need to call seterror yourself */
 #define SETERROR( utils, msg ) (utils)->seterror( (utils)->conn, 0, (msg) )
