@@ -1,10 +1,10 @@
 /* testsuite.c -- Stress the library a little
  * Rob Siemborski
  * Tim Martin
- * $Id: testsuite.c,v 1.31 2002/12/09 21:01:09 rjs3 Exp $
+ * $Id: testsuite.c,v 1.33 2003/03/28 19:59:45 rjs3 Exp $
  */
 /* 
- * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -455,6 +455,11 @@ int good_getopt(void *context __attribute__((unused)),
 	if (len)
 	    *len = strlen("auxprop");
 	return SASL_OK;
+    } else if (!strcmp(option, "auxprop_plugin")) {
+	*result = "sasldb";
+	if (len)
+	    *len = strlen("sasldb");
+	return SASL_OK;
     } else if (!strcmp(option, "sasldb_path")) {
 	*result = "./sasldb";
 	if (len)
