@@ -2,7 +2,7 @@
  * Ryan Troll
  */
 /* 
- * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -50,7 +50,7 @@
 #define PACKAGE "cyrus-sasl"
 
 /* Our version */
-#define VERSION "2.1.12"
+#define VERSION "2.1.13"
 
 /* Registry key that contains the locations of the plugins */
 #define SASL_ROOT_KEY "SOFTWARE\\Carnegie Mellon\\Project Cyrus\\SASL Library"
@@ -152,6 +152,6 @@ struct sockaddr_storage {
 #define	NI_WITHSCOPEID	0
 #endif
 
-#define HAVE_TIME_H
+#include <time.h>
 
 #endif /* CONFIG_H */
